@@ -1,33 +1,49 @@
-#' Get summaries of PanelEstimate objects/calculations
+#' Get summaries of PanelEstimate objects and calculations
 #' 
 #' 
 #' \code{summary.PanelEstimate} takes an object returned by
 #' \code{PanelEstimate}, and returns a summary table of point
 #' estimates and confidence intervals
 #'
-#' @param object A PanelEstimate object
-#' @param verbose logical indicating whether or not output should be printed in an expanded form. Default is TRUE
+#' @param object A \code{PanelEstimate} object
+#' @param verbose logical indicating whether or not output should be printed in an expanded form. Default is FALSE
+#' @param confidence.level Confidence level to be used for confidence interval calculations. Must be numeric between 0 and 1. If NULL, confidence level from \code{PanelEstimate()} specification is used. 
 #' @param bias.corrected logical indicating whether or not bias corrected estimates should be provided. Default is FALSE. This argument only applies for standard errors calculated with the bootstrap. 
 #' @param ... optional additional arguments. Currently, no additional arguments are supported. 
 #' @examples
 #' dem.sub <- dem[dem[, "wbcode2"] <= 100, ]
+#' dem.sub.panel <- PanelData(dem.sub, "wbcode2", "year", "dem", "y")
 #' # create subset of data for simplicity
-#' PM.results <- PanelMatch(lag = 4, time.id = "year", unit.id = "wbcode2", 
-#'                          treatment = "dem", refinement.method = "ps.weight", 
-#'                          data = dem.sub, match.missing = TRUE, 
+#' PM.results <- PanelMatch(panel.data = dem.sub.panel, lag = 4, 
+#'                          refinement.method = "ps.match", 
+#'                          match.missing = TRUE, 
 #'                          covs.formula = ~ tradewb,
 #'                          size.match = 5, qoi = "att",
-#'                          outcome.var = "y", lead = 0:4, forbid.treatment.reversal = FALSE)
-#' PE.results <- PanelEstimate(sets = PM.results, data = dem.sub, se.method = "unconditional")
+#'                          lead = 0:4, 
+#'                          forbid.treatment.reversal = FALSE)
+#' PE.results <- PanelEstimate(sets = PM.results, 
+#'                             panel.data = dem.sub.panel, 
+#'                             se.method = "unconditional")
 #' summary(PE.results)
-#' 
+#' summary(PE.results, confidence.level = .9)
 #'
-#' 
 #' @method summary PanelEstimate
 #' @export
-summary.PanelEstimate <- function(object, verbose = TRUE, 
+summary.PanelEstimate <- function(object, 
+                                  confidence.level = NULL,
+                                  verbose = FALSE, 
                                   bias.corrected = FALSE, ...) {
   
+  if (is.null(confidence.level))
+  {
+    confidence.level <- object$confidence.level
+  }
+  
+  confidence.check <- is.numeric(confidence.level) && confidence.level >= 0 && confidence.level <= 1
+  if (!confidence.check) {
+    stop("confidence level should be a number between 0 and 1. Please specify via argument or use default from PanelEstimate object.")
+  } 
+  
   if (verbose)
   {
     if (object$qoi == "ate")
@@ -91,21 +107,21 @@ summary.PanelEstimate <- function(object, verbose = TRUE,
   if (bias.corrected && identical(object$se.method, "bootstrap"))
   {
     if (!identical(object$se.method, "bootstrap")) stop("bias corrected estimates only available for bootstrap method currently")
-    a1 <- t(as.data.frame(object$estimates))
+    a1 <- t(as.data.frame(object$estimate))
     a2 <- apply(object$bootstrapped.estimates, 2, sd, na.rm = TRUE)
     a3 <- apply(object$bootstrapped.estimates, 2, 
           quantile, 
-          probs = c( (1 - object$confidence.level)/2, 
-                     object$confidence.level + (1 - object$confidence.level)/2 ), 
+          probs = c( (1 - confidence.level)/2, 
+                     confidence.level + (1 - confidence.level)/2 ), 
           na.rm = TRUE)
-    a4 <- 2*object$estimates - colMeans(object$bootstrapped.estimates, na.rm = TRUE)
+    a4 <- 2*object$estimate - colMeans(object$bootstrapped.estimates, na.rm = TRUE)
     a5 <- apply( (2*matrix(nrow = object$bootstrap.iterations, 
-                     ncol = length(object$estimates), 
-                     object$estimates, byrow = TRUE) - object$bootstrapped.estimates), 
+                     ncol = length(object$estimate), 
+                     object$estimate, byrow = TRUE) - object$bootstrapped.estimates), 
            2, 
            quantile, 
-           probs = c((1 - object$confidence.level)/2, 
-                     object$confidence.level + (1 - object$confidence.level)/2), 
+           probs = c((1 - confidence.level)/2, 
+                     confidence.level + (1 - confidence.level)/2), 
            na.rm = TRUE)
     df <- rbind(a1, # point estimate
                 a2, # bootstrap se
@@ -116,29 +132,29 @@ summary.PanelEstimate <- function(object, verbose = TRUE,
                 a5) # bc percentile confidence.level)
     
     rownames(df) <- c("estimate", "std.error", 
-                      paste0((1 - object$confidence.level)/2 * 100, "%"),
-                      paste0( (object$confidence.level + (1 - object$confidence.level)/2) * 100, "%"),
+                      paste0((1 - confidence.level)/2 * 100, "%"),
+                      paste0( (confidence.level + (1 - confidence.level)/2) * 100, "%"),
                       "estimate(bias corrected)", 
-                      paste0((1 - object$confidence.level)/2 * 100, "%", "(bias corrected)"),
-                      paste0((object$confidence.level + (1 - object$confidence.level)/2) * 100, "%", "(bias corrected)"))
+                      paste0((1 - confidence.level)/2 * 100, "%", "(bias corrected)"),
+                      paste0((confidence.level + (1 - confidence.level)/2) * 100, "%", "(bias corrected)"))
   }
   else
   {
     if ( identical(object$se.method, "bootstrap") )
     {
-      a1 <- t(as.data.frame(object$estimates))
+      a1 <- t(as.data.frame(object$estimate))
       a2 <- apply(object$bootstrapped.estimates, 2, sd, na.rm = TRUE)
       a3 <- apply(object$bootstrapped.estimates, 2, quantile, 
-            probs = c( (1 - object$confidence.level)/2, 
-                       object$confidence.level + (1 - object$confidence.level)/2 ), 
+            probs = c( (1 - confidence.level)/2, 
+                       confidence.level + (1 - confidence.level)/2 ), 
             na.rm = TRUE)
       df <- rbind(a1, # point estimate
                   a2, # bootstrap se
                   a3) # Efron & Tibshirani 1993 p170 - 171
       
       rownames(df) <- c("estimate", "std.error", 
-                        paste0((1 - object$confidence.level)/2 * 100, "%"),
-                        paste0( (object$confidence.level + (1 - object$confidence.level)/2) * 100, "%"))
+                        paste0((1 - confidence.level)/2 * 100, "%"),
+                        paste0( (confidence.level + (1 - confidence.level)/2) * 100, "%"))
       tdf <- t(df)
       if (!verbose) return(t(df))
       return(list("summary" = tdf, 
@@ -150,21 +166,21 @@ summary.PanelEstimate <- function(object, verbose = TRUE,
              identical(object$se.method, "unconditional")) 
     {
       
-      critical.vals <- rep(qnorm( (1 - object$confidence.level) / 2), 2) * c(1, -1)
+      critical.vals <- rep(qnorm( (1 - confidence.level) / 2), 2) * c(1, -1)
       
       quants <- mapply(FUN = function(x, y) x + critical.vals * y,
-                       x = object$estimates, 
+                       x = object$estimate, 
                        y = object$standard.error,
                        SIMPLIFY = FALSE)
       qts <- do.call(rbind, quants)
-      df <- data.frame(estimate = object$estimates,
+      df <- data.frame(estimate = object$estimate,
                        std.error = object$standard.error)
       
       tdf <- cbind(df, qts)
       colnames(tdf) <- c("estimate", "std.error", 
-                         paste0((1 - object$confidence.level)/2 * 100, "%"),
-                         paste0( (object$confidence.level + (1 - object$confidence.level)/2) * 100, "%"))
-      rownames(tdf) <- names(object$estimates)
+                         paste0((1 - confidence.level)/2 * 100, "%"),
+                         paste0( (confidence.level + (1 - confidence.level)/2) * 100, "%"))
+      rownames(tdf) <- names(object$estimate)
       if (!verbose) return(tdf)
       return(list("summary" = tdf, 
                   "lag" = lag, 
@@ -192,18 +208,23 @@ summary.PanelEstimate <- function(object, verbose = TRUE,
 #' @param ylim default is NULL. This is the same argument as the standard argument for \code{plot()}
 #' @param pch default is NULL. This is the same argument as the standard argument for \code{plot()}
 #' @param cex default is NULL. This is the same argument as the standard argument for \code{plot()}
+#' @param confidence.level confidence.level Confidence level to be used for confidence interval calculations. Must be numeric between 0 and 1. If NULL, confidence level from \code{PanelEstimate()} specification is used. 
 #' @param bias.corrected logical indicating whether or not bias corrected estimates should be plotted Default is FALSE. This argument only applies for standard errors calculated with the bootstrap. 
 #' @param ... Additional optional arguments to be passed to \code{plot()}.
 #' @examples
 #' dem.sub <- dem[dem[, "wbcode2"] <= 100, ]
+#' dem.sub.panel <- PanelData(dem.sub, "wbcode2", "year", "dem", "y")
 #' # create subset of data for simplicity
-#' PM.results <- PanelMatch(lag = 4, time.id = "year", unit.id = "wbcode2", 
-#'                          treatment = "dem", refinement.method = "mahalanobis", 
-#'                          data = dem.sub, match.missing = TRUE, 
+#' PM.results <- PanelMatch(panel.data = dem.sub.panel, lag = 4, 
+#'                          refinement.method = "ps.match", 
+#'                          match.missing = TRUE, 
 #'                          covs.formula = ~ tradewb,
 #'                          size.match = 5, qoi = "att",
-#'                          outcome.var = "y", lead = 0:4, forbid.treatment.reversal = FALSE)
-#' PE.results <- PanelEstimate(sets = PM.results, data = dem.sub, se.method = "unconditional")
+#'                          lead = 0:4, 
+#'                          forbid.treatment.reversal = FALSE)
+#' PE.results <- PanelEstimate(sets = PM.results, 
+#'               panel.data = dem.sub.panel, 
+#'               se.method = "unconditional")
 #' plot(PE.results)
 #'
 #' @method plot PanelEstimate
@@ -215,12 +236,15 @@ plot.PanelEstimate <- function(x,
                                ylim = NULL, 
                                pch = NULL,
                                cex = NULL,
+                               confidence.level = NULL,
                                bias.corrected = FALSE,
                                ...)
 {
   pe.object <- x
   plot.data <- summary(pe.object, 
-                       verbose = FALSE, bias.corrected = bias.corrected)
+                       verbose = FALSE, 
+                       bias.corrected = bias.corrected, 
+                       confidence.level = confidence.level)
   
   if (is.null(ylim))
   {
@@ -246,6 +270,37 @@ plot.PanelEstimate <- function(x,
   graphics::segments(1:(nrow(plot.data)), 
                      plot.data[,3], 
                      1:(nrow(plot.data)), 
-                     plot.data[,4])
-  graphics::abline(h = 0, lty = "dashed")
+                     plot.data[,4],
+                     ...)
+  graphics::abline(h = 0, lty = "dashed",
+                   ...)
+}
+
+#' Print point estimates and standard errors 
+#' @param x \code{PanelEstimate} object
+#' @param ... additional arguments to be passed to \code{print.data.frame()}
+#' @examples
+#' dem.sub <- dem[dem[, "wbcode2"] <= 100, ]
+#' dem.sub.panel <- PanelData(dem.sub, "wbcode2", "year", "dem", "y")
+#' # create subset of data for simplicity
+#' PM.results <- PanelMatch(panel.data = dem.sub.panel, lag = 4, 
+#'                          refinement.method = "ps.match", 
+#'                          match.missing = TRUE, 
+#'                          covs.formula = ~ tradewb,
+#'                          size.match = 5, qoi = "att",
+#'                          lead = 0:4, 
+#'                          forbid.treatment.reversal = FALSE)
+#' PE.results <- PanelEstimate(sets = PM.results, 
+#'               panel.data = dem.sub.panel, 
+#'               se.method = "unconditional")
+#' print(PE.results)
+#' @method print PanelEstimate
+#' @export
+print.PanelEstimate <- function(x, ...)
+{
+  
+  cat("Point estimates:\n")
+  print(x[["estimate"]])
+  cat("Standard errors:\n")
+  print(x[["standard.error"]])
 }
